@@ -1,6 +1,7 @@
 import logging
-from typing import Any, Mapping, Sequence
+from typing import Any, Mapping, Optional, Sequence
 
+from markdown_it.common.normalize_url import unescape_string
 from markdown_it.token import Token
 
 from mdformat.renderer import _container_renderers, _token_renderers
@@ -24,12 +25,14 @@ class MDRenderer:
         """__init__ must have `parser` parameter for markdown-it-py
         compatibility."""
 
-    def render(
+    def render(  # noqa: C901
         self,
         tokens: Sequence[Token],
         options: Mapping[str, Any],
         env: dict,
         *,
+        start: int = 0,
+        stop: Optional[int] = None,
         finalize: bool = True,
         _recursion_level: int = 0,
     ) -> str:
@@ -39,6 +42,8 @@ class MDRenderer:
             tokens: A list of block tokens to render
             options: Params of parser instance
             env: Additional data from parsed input
+            start: Start rendering from this index
+            stop: Stop rendering before this index
             finalize: replace markers and write references
         """
         assert _recursion_level in {
@@ -47,9 +52,10 @@ class MDRenderer:
         }, "There should be no more than one level of recursion in tokens"
         text_stack = [""]
 
-        i = -1
-        tokens_length = len(tokens)
-        while (i + 1) < tokens_length:
+        i = start - 1
+        if stop is None:
+            stop = len(tokens)
+        while (i + 1) < stop:
             i += 1
             token = tokens[i]
 
@@ -125,8 +131,13 @@ class MDRenderer:
         text = ""
         for label in sorted(env.get("used_refs", [])):
             ref = env["references"][label]
-            item = f"[{label.lower()}]: {ref['href']}"
-            if ref["title"]:
-                item += f' "{ref["title"]}"'
+            destination = ref["href"] if ref["href"] else "<>"
+            destination = unescape_string(destination)
+            item = f"[{label.lower()}]: {destination}"
+            title = ref["title"]
+            if title:
+                title = unescape_string(title)
+                title = title.replace('"', '\\"')
+                item += f' "{title}"'
             text += item + "\n"
         return text.rstrip()
